/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.network;

import java.io.*;
import javax.swing.table.DefaultTableModel;
import melodia.player.gui.MainFrame;

/**
 *
 * @author Wyverald
 */
public class DownloadRequest implements Request<Void> {
  protected String title;
  protected String artist;
  protected String album;
  protected File dest;

  public DownloadRequest(File dest, String title, String artist, String album) {
    this.title = title;
    this.artist = artist;
    this.album = album;
    this.dest = dest;
  }

  @Override
  public void send(DataOutputStream os) throws IOException {
    os.writeUTF("download");
    os.writeUTF(title);
    os.writeUTF(artist);
    os.writeUTF(album);
  }

  @Override
  public Void parseResponse(DataInputStream is) throws IOException {
    melodia.util.FileRedirector.dataInputStreamToFile(dest, is);
    melodia.util.FileRedirector.dataInputStreamToFile(
        melodia.player.Lyrics.findLyricsFileFromSongFile(dest), is);
    return null;
  }

  @Override
  public void updateView(MainFrame frame, DefaultTableModel model, Void result) {
    frame.serverInfo("Download complete: " + dest.getName());
  }
  
}
