/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.network;

import java.io.*;
import javax.swing.table.DefaultTableModel;
import melodia.player.gui.MainFrame;

/**
 *
 * @author Wyverald
 */
public class ListRequest implements Request<ListResponse> {
  public static int DEFAULT_PAGE_SIZE = 10;
  private String query;
  private int offset, count;
  
  public ListRequest() {
    this("", 1, DEFAULT_PAGE_SIZE);
  }
  
  public ListRequest(String query) {
    this(query, 1, DEFAULT_PAGE_SIZE);
  }
  
  public ListRequest(int offset, int count) {
    this("", offset, count);
  }
  
  public ListRequest(String query, int offset, int count) {
    this.query = query;
    this.offset = offset;
    this.count = count;
  }

  @Override
  public void send(DataOutputStream os) throws IOException {
    os.writeUTF("list");
    os.writeUTF(query);
    os.writeInt(offset);
    os.writeInt(count);
  }

  @Override
  public ListResponse parseResponse(DataInputStream is) throws IOException {
    int total = is.readInt();
    int from = is.readInt();
    int to = is.readInt();
    
    String[][] list = new String[to - from + 1][];
    
    for (int i = 0; i < list.length; i++) {
      list[i] = is.readUTF().split("#");
    }
    return new ListResponse(list, total, from, to);
  }

  @Override
  public void updateView(MainFrame frame, DefaultTableModel model, ListResponse result) {
    model.setRowCount(0);
    for (String[] song : result.list) {
      model.addRow(song);
    }
    frame.serverInfo(String.format("Displaying %d - %d of %d results",
        result.from, result.to, result.total));
    frame.setListPages(result.from, result.total);
  }
  
}
