/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.network;

import java.io.*;
import java.util.concurrent.ExecutionException;
import javax.swing.SwingWorker;
import java.net.*;
import javax.swing.table.*;
import melodia.player.gui.MainFrame;

/**
 *
 * @author Wyverald
 */
public class RequestHandler<T> extends SwingWorker<T, Void> {
  private Socket socket;
  private MainFrame frame;
  private DefaultTableModel table;
  private Request<T> request;
  
  public RequestHandler(MainFrame frame, TableModel tableModel,
      Request<T> req, String host, int port)
      throws UnknownHostException, IOException {
    this.socket = new Socket(host, port);
    this.frame = frame;
    this.table = (DefaultTableModel) tableModel;
    this.request = req;
  }
  
  @Override
  protected T doInBackground() {
    try {
      DataOutputStream os = new DataOutputStream(socket.getOutputStream());
      DataInputStream is = new DataInputStream(socket.getInputStream());
      
      request.send(os);
      os.flush();
      T response = request.parseResponse(is);
      
      os.close();
      is.close();
      socket.close();
      
      return response;
    } catch (IOException ex) {
      ex.printStackTrace();
      return null;
    }
  }
  
  @Override
  public void done() {
    try {
      request.updateView(frame, table, get());
    } catch (InterruptedException ex) {
    } catch (ExecutionException ex) {
      frame.serverError("Request error.");
      ex.getCause().printStackTrace();
    }
  }
}
