/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.network;

import java.io.*;
import java.net.SocketException;
import javax.swing.table.DefaultTableModel;
import melodia.player.gui.MainFrame;

/**
 *
 * @author Wyverald
 */
public class UploadRequest implements Request<Boolean> {
  protected String title;
  protected String artist;
  protected String album;
  protected File src;
  protected boolean failed = false;

  public UploadRequest(File src, String title, String artist, String album) {
    this.title = title;
    this.artist = artist;
    this.album = album;
    this.src = src;
  }

  @Override
  public void send(DataOutputStream os) throws IOException {
    os.writeUTF("upload");
    os.writeUTF(title);
    os.writeUTF(artist);
    os.writeUTF(album);
    
    try {
      melodia.util.FileRedirector.fileToDataOutputStream(src, os);
      melodia.util.FileRedirector.fileToDataOutputStream(
          melodia.player.Lyrics.findLyricsFileFromSongFile(src), os);
    } catch (SocketException ex) {
      failed = true;
    }
  }

  @Override
  public Boolean parseResponse(DataInputStream is) throws IOException {
    return !failed && is.readBoolean();
  }

  @Override
  public void updateView(MainFrame frame, DefaultTableModel model, Boolean result) {
    if (result.booleanValue()) {
      frame.serverRefresh();
    } else {
      frame.serverError("Song already on server.");
    }
  }
  
}
