/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player;

import melodia.player.gui.PlaylistButton;
import java.io.*;
import java.util.*;
import org.json.*;

/**
 *
 * @author Wyverald
 */
public class Playlist {
  private String name;
  protected ArrayList<File> files;
  private PlaylistButton button;
  
  public Playlist(String name) {
    this.name = name;
    files = new ArrayList<File>();
    button = new PlaylistButton(this);
  }
  
  public Playlist(String name, ArrayList<File> arr) {
    this(name);
    this.files = (ArrayList<File>) arr.clone();
    button.updateView();
  }
  
  public Playlist(String name, JSONArray arr) {
    this(name);
    for (int i = 0; i < arr.length(); i++) {
      try {
        files.add(new File(arr.getString(i)));
      } catch (JSONException ex) {
        System.err.println("JSONException: " + ex.getMessage());
      }
    }
    button.updateView();
  }
  
  public String getName() {
    return name;
  }
  
  public void setName(String name) {
    this.name = name;
    button.updateView();
  }
  
  public PlaylistButton getButton() {
    return button;
  }
  
  public ArrayList<File> getFiles() {
    return files;
  }
  
  public boolean isEmpty() {
    return files.isEmpty();
  }
  
  public int size() {
    return files.size();
  }
  
  public File[] getFileArray() {
    File[] arr = new File[files.size()];
    return files.toArray(arr);
  }
  
  public JSONArray getJSONArray() {
    JSONArray arr = new JSONArray();
    for (Iterator<File> it = files.iterator(); it.hasNext();) {
      File file = it.next();
      arr.put(file.getAbsolutePath());
    }
    return arr;
  }
}
