/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player.gui;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import melodia.player.Lyrics;
import melodia.player.Lyrics.Line;

/**
 *
 * @author Wyverald
 */
public class LyricsScroller {
  private ArrayList<Line> lines;
  private Lyrics lyrics;
  private JScrollPane scroll;
  private JPanel panel;
  private JLabel[] labels;
  
  private final int RIGID_AREA_HEIGHT = 500;
  private /*final*/ int LABEL_HEIGHT;

  public LyricsScroller(JScrollPane scrollPane, JPanel pnlLyrics, Lyrics lyrics) {
    this.lyrics = lyrics;
    lines = lyrics.getLines();
    scroll = scrollPane;
    panel = pnlLyrics;
    labels = new JLabel[lines.size()];
    
    Font lyricsFont = new Font("Microsoft YaHei", Font.PLAIN, 12);
    
    panel.add(Box.createRigidArea(new Dimension(0, RIGID_AREA_HEIGHT)));
    for (int i = 0; i < lines.size(); i++) {
      labels[i] = new JLabel(lines.get(i).lyrics);
      if (labels[i].getText().isEmpty()) labels[i].setText(" ");
      labels[i].setAlignmentX(0.5f);
      labels[i].setHorizontalAlignment(JLabel.CENTER);
      labels[i].setForeground(Color.GRAY);
      labels[i].setFont(lyricsFont);
      panel.add(labels[i]);
    }
    panel.add(Box.createRigidArea(new Dimension(0, RIGID_AREA_HEIGHT)));
    
    panel.validate();
    LABEL_HEIGHT = labels[0].getHeight();
  }

  public void update(double currentTime) {
    int index = lyrics.getLineIndexFromTime(currentTime);
    
    if (index < 0) {
      panel.setLocation(0, scroll.getHeight() / 2 - RIGID_AREA_HEIGHT);
      return;
    }
    
    // Scroll to the correct offset.
    int curLineOffset = (int) ((currentTime - lines.get(index).time) /
        lines.get(index).length * LABEL_HEIGHT);
    int totalOffset = RIGID_AREA_HEIGHT + LABEL_HEIGHT * index + curLineOffset;
    panel.setLocation(0, scroll.getHeight() / 2 - totalOffset);
    
    
    // Set the color of every label.
    for (int i = 0; i < lines.size(); i++) {
      labels[i].setForeground(i == index ? Color.BLACK : Color.GRAY);
    }
  }
  
}
