/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/*
 * MainFrame.java
 *
 * Created on 2011-8-26, 14:26:26
 */
package melodia.player.gui;

import java.net.UnknownHostException;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.event.*;
import melodia.network.*;
import melodia.player.*;
import melodia.util.Config;
import org.json.*;

/**
 *
 * @author Wyverald
 */
public class MainFrame extends javax.swing.JFrame implements PlayerListener,
    PlaylistListener {

  private JFileChooser fileChooser;
  private Timer progressTimer;
  private Timer statusTimer;
  private boolean suppressUpdate;
  private PlayerManager player;
  private PlaylistManager playlist;
  private int statusType = 0;
  private int loopMode = 0;
  private int serverListOffset = 0, serverListTotal = 0;
  private boolean randomized = false, connected = false;
  private LyricsScroller lyricsScroller;

  /** Creates new form MainFrame */
  public MainFrame() {
    initComponents();
    replaceDivider();
    makeHoverButtons();
    decorateWindow();
    createFileChooser();
    setConnected(false);
    progressTimer = new Timer(100, new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent ae) {
        progressTimerTick(ae);
      }
    });
    statusTimer = new Timer(5000, new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent ae) {
        statusType = (statusType + 1) % 5;
        statusTimerTick();
      }
    });
    player = new PlayerManager(this);
    playlist = new PlaylistManager(this);
    suppressUpdate = false;
    loadGUIConfig();
    playlist.loadSavedPlaylists();
    statusTimerTick();
    statusTimer.start();
  }
  
  public PlayerManager getPlayerManager() { return player; }
  public PlaylistManager getPlaylistManager() { return playlist; }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        popupMenu = new javax.swing.JPopupMenu();
        menuAlwaysOnTop = new javax.swing.JCheckBoxMenuItem();
        menuMiniMode = new javax.swing.JMenuItem();
        menuExpanded = new javax.swing.JMenuItem();
        menuAbout = new javax.swing.JMenuItem();
        pnlMain = new javax.swing.JPanel();
        pnlTitleBarContainer = new javax.swing.JPanel();
        pnlTitleBar = new javax.swing.JPanel();
        pnlWindowControls = new javax.swing.JPanel();
        lblResize = new javax.swing.JLabel();
        btnClose = new javax.swing.JButton();
        btnMinimize = new javax.swing.JButton();
        btnMenu = new javax.swing.JButton();
        filler11 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        lblAppTitle = new javax.swing.JLabel();
        filler3 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        pnlView = new javax.swing.JPanel();
        tlgPlaylist = new javax.swing.JToggleButton();
        tlgLyrics = new javax.swing.JToggleButton();
        tlgServer = new javax.swing.JToggleButton();
        lblTitleBackground = new javax.swing.JLabel();
        pnlTop = new javax.swing.JSplitPane();
        pnlPlaylistScroll = new javax.swing.JScrollPane();
        pnlPlaylist = new javax.swing.JPanel();
        btnAddSong = new javax.swing.JButton();
        btnClearPlaylist = new javax.swing.JButton();
        pnlPlaylistsScroll = new javax.swing.JScrollPane();
        pnlPlaylists = new javax.swing.JPanel();
        btnSavePlaylist = new javax.swing.JButton();
        pnlProgress = new javax.swing.JPanel();
        sldProgress = new javax.swing.JSlider();
        lblDuration = new javax.swing.JLabel();
        lblTime = new javax.swing.JLabel();
        pnlControls = new javax.swing.JPanel();
        pnlInfo = new javax.swing.JPanel();
        lblStatus = new javax.swing.JLabel();
        lblTitle = new javax.swing.JLabel();
        lblExtraInfo = new javax.swing.JLabel();
        pnlPlayControls = new javax.swing.JPanel();
        filler1 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        btnPreviousRewind = new javax.swing.JButton();
        btnPlayPause = new javax.swing.JButton();
        btnNext = new javax.swing.JButton();
        filler4 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        pnlMiscControls = new javax.swing.JPanel();
        filler2 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        btnLoopMode = new javax.swing.JButton();
        btnOrder = new javax.swing.JButton();
        btnVolume = new javax.swing.JButton();
        sldVolume = new javax.swing.JSlider();
        pnlBottom = new javax.swing.JSplitPane();
        pnlLyricsScroll = new javax.swing.JScrollPane();
        pnlLyrics = new javax.swing.JPanel();
        pnlServer = new javax.swing.JPanel();
        pnlServerControls = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        txtHost = new javax.swing.JTextField();
        filler5 = new javax.swing.Box.Filler(new java.awt.Dimension(8, 0), new java.awt.Dimension(8, 0), new java.awt.Dimension(8, 0));
        jLabel2 = new javax.swing.JLabel();
        txtPort = new javax.swing.JFormattedTextField();
        filler8 = new javax.swing.Box.Filler(new java.awt.Dimension(8, 0), new java.awt.Dimension(8, 0), new java.awt.Dimension(8, 0));
        jLabel3 = new javax.swing.JLabel();
        txtUsername = new javax.swing.JTextField();
        btnConnect = new javax.swing.JButton();
        pnlServerScroll = new javax.swing.JScrollPane();
        tblServer = new javax.swing.JTable();
        pnlServerListControls = new javax.swing.JPanel();
        filler6 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 32767));
        btnServerListPrevious = new javax.swing.JButton();
        btnServerListRefresh = new javax.swing.JButton();
        btnServerListNext = new javax.swing.JButton();
        filler9 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 8), new java.awt.Dimension(0, 8), new java.awt.Dimension(0, 8));
        btnServerListDownload = new javax.swing.JButton();
        btnServerListPlay = new javax.swing.JButton();
        filler7 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 32767));
        pnlServerStatus = new javax.swing.JPanel();
        lblServerStatus = new javax.swing.JLabel();
        filler10 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        jLabel4 = new javax.swing.JLabel();
        txtServerListSearch = new javax.swing.JTextField();

        popupMenu.setInvoker(btnMenu);

        menuAlwaysOnTop.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_T, java.awt.event.InputEvent.CTRL_MASK));
        menuAlwaysOnTop.setMnemonic('T');
        menuAlwaysOnTop.setText("Always on top");
        menuAlwaysOnTop.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                menuAlwaysOnTopItemStateChanged(evt);
            }
        });
        popupMenu.add(menuAlwaysOnTop);

        menuMiniMode.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_M, java.awt.event.InputEvent.CTRL_MASK));
        menuMiniMode.setMnemonic('M');
        menuMiniMode.setText("Mini Mode");
        menuMiniMode.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuMiniModeActionPerformed(evt);
            }
        });
        popupMenu.add(menuMiniMode);

        menuExpanded.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.event.InputEvent.CTRL_MASK));
        menuExpanded.setMnemonic('X');
        menuExpanded.setText("Expanded Mode");
        menuExpanded.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuExpandedActionPerformed(evt);
            }
        });
        popupMenu.add(menuExpanded);

        menuAbout.setMnemonic('A');
        menuAbout.setText("About");
        menuAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuAboutActionPerformed(evt);
            }
        });
        popupMenu.add(menuAbout);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Melodia");
        setIconImage(new ImageIcon(getClass().getResource("/melodia/player/gui/resources/unknown-album.png")).getImage());
        setMinimumSize(new java.awt.Dimension(300, 50));
        setUndecorated(true);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                formWindowClosed(evt);
            }
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        pnlMain.setMinimumSize(new java.awt.Dimension(0, 0));
        pnlMain.setLayout(new javax.swing.BoxLayout(pnlMain, javax.swing.BoxLayout.PAGE_AXIS));

        pnlTitleBarContainer.setAlignmentX(0.0F);
        pnlTitleBarContainer.setMinimumSize(new java.awt.Dimension(672, 29));
        pnlTitleBarContainer.setPreferredSize(new java.awt.Dimension(672, 30));
        pnlTitleBarContainer.setLayout(new java.awt.GridBagLayout());

        pnlTitleBar.setAlignmentX(0.0F);
        pnlTitleBar.setOpaque(false);
        pnlTitleBar.setPreferredSize(new java.awt.Dimension(672, 30));
        pnlTitleBar.setLayout(new javax.swing.BoxLayout(pnlTitleBar, javax.swing.BoxLayout.LINE_AXIS));

        pnlWindowControls.setOpaque(false);
        pnlWindowControls.setLayout(new javax.swing.BoxLayout(pnlWindowControls, javax.swing.BoxLayout.LINE_AXIS));

        lblResize.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/border-up.png"))); // NOI18N
        lblResize.setAlignmentY(1.0F);
        lblResize.setCursor(new java.awt.Cursor(java.awt.Cursor.NW_RESIZE_CURSOR));
        pnlWindowControls.add(lblResize);

        btnClose.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/cross-circle.png"))); // NOI18N
        btnClose.setBorder(null);
        btnClose.setContentAreaFilled(false);
        btnClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCloseActionPerformed(evt);
            }
        });
        pnlWindowControls.add(btnClose);

        btnMinimize.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/navigation-270.png"))); // NOI18N
        btnMinimize.setBorder(null);
        btnMinimize.setContentAreaFilled(false);
        btnMinimize.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMinimizeActionPerformed(evt);
            }
        });
        pnlWindowControls.add(btnMinimize);

        btnMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/plus-circle.png"))); // NOI18N
        btnMenu.setBorder(null);
        btnMenu.setContentAreaFilled(false);
        btnMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMenuActionPerformed(evt);
            }
        });
        pnlWindowControls.add(btnMenu);

        pnlTitleBar.add(pnlWindowControls);
        pnlTitleBar.add(filler11);

        lblAppTitle.setFont(new java.awt.Font("Georgia", 1, 18)); // NOI18N
        lblAppTitle.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        lblAppTitle.setText("Melodia");
        pnlTitleBar.add(lblAppTitle);
        pnlTitleBar.add(filler3);

        pnlView.setOpaque(false);
        pnlView.setLayout(new javax.swing.BoxLayout(pnlView, javax.swing.BoxLayout.LINE_AXIS));

        tlgPlaylist.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/notebook.png"))); // NOI18N
        tlgPlaylist.setSelected(true);
        tlgPlaylist.setToolTipText("Playlist panel");
        tlgPlaylist.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                tlgPlaylistItemStateChanged(evt);
            }
        });
        pnlView.add(tlgPlaylist);

        tlgLyrics.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/balloon-quotation.png"))); // NOI18N
        tlgLyrics.setSelected(true);
        tlgLyrics.setToolTipText("Lyrics panel");
        tlgLyrics.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                tlgLyricsItemStateChanged(evt);
            }
        });
        pnlView.add(tlgLyrics);

        tlgServer.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/network-cloud.png"))); // NOI18N
        tlgServer.setSelected(true);
        tlgServer.setToolTipText("Server panel");
        tlgServer.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                tlgServerItemStateChanged(evt);
            }
        });
        pnlView.add(tlgServer);

        pnlTitleBar.add(pnlView);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        pnlTitleBarContainer.add(pnlTitleBar, gridBagConstraints);

        lblTitleBackground.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/title-background.png"))); // NOI18N
        lblTitleBackground.setMinimumSize(new java.awt.Dimension(700, 25));
        lblTitleBackground.setPreferredSize(new java.awt.Dimension(700, 30));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        pnlTitleBarContainer.add(lblTitleBackground, gridBagConstraints);

        pnlMain.add(pnlTitleBarContainer);

        pnlTop.setDividerLocation(450);
        pnlTop.setDividerSize(25);
        pnlTop.setResizeWeight(0.8);
        pnlTop.setPreferredSize(new java.awt.Dimension(11, 160));

        pnlPlaylistScroll.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        pnlPlaylistScroll.setMinimumSize(new java.awt.Dimension(100, 23));

        pnlPlaylist.setMinimumSize(new java.awt.Dimension(0, 0));
        pnlPlaylist.setLayout(new javax.swing.BoxLayout(pnlPlaylist, javax.swing.BoxLayout.LINE_AXIS));

        btnAddSong.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/music--plus.png"))); // NOI18N
        btnAddSong.setToolTipText("Add song to current playlist");
        btnAddSong.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        btnAddSong.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddSongActionPerformed(evt);
            }
        });
        pnlPlaylist.add(btnAddSong);

        btnClearPlaylist.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/bin-full.png"))); // NOI18N
        btnClearPlaylist.setToolTipText("Clear current playlist");
        btnClearPlaylist.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        btnClearPlaylist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnClearPlaylistActionPerformed(evt);
            }
        });
        pnlPlaylist.add(btnClearPlaylist);

        pnlPlaylistScroll.setViewportView(pnlPlaylist);

        pnlTop.setLeftComponent(pnlPlaylistScroll);

        pnlPlaylistsScroll.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        pnlPlaylistsScroll.setMinimumSize(new java.awt.Dimension(50, 23));

        pnlPlaylists.setMinimumSize(new java.awt.Dimension(0, 0));
        pnlPlaylists.setLayout(new javax.swing.BoxLayout(pnlPlaylists, javax.swing.BoxLayout.LINE_AXIS));

        btnSavePlaylist.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/notebook--arrow.png"))); // NOI18N
        btnSavePlaylist.setToolTipText("Save current playlist");
        btnSavePlaylist.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        btnSavePlaylist.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSavePlaylistActionPerformed(evt);
            }
        });
        pnlPlaylists.add(btnSavePlaylist);

        pnlPlaylistsScroll.setViewportView(pnlPlaylists);

        pnlTop.setRightComponent(pnlPlaylistsScroll);

        pnlMain.add(pnlTop);

        pnlProgress.setAlignmentX(0.0F);
        pnlProgress.setLayout(new java.awt.GridBagLayout());

        sldProgress.setMajorTickSpacing(100);
        sldProgress.setMaximum(500);
        sldProgress.setMinorTickSpacing(50);
        sldProgress.setValue(0);
        sldProgress.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                sldProgressStateChanged(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlProgress.add(sldProgress, gridBagConstraints);

        lblDuration.setText("00:00");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlProgress.add(lblDuration, gridBagConstraints);

        lblTime.setText("00:00");
        lblTime.setToolTipText("");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 5);
        pnlProgress.add(lblTime, gridBagConstraints);

        pnlMain.add(pnlProgress);

        pnlControls.setAlignmentX(0.0F);
        pnlControls.setLayout(new java.awt.GridLayout(1, 3));

        pnlInfo.setLayout(new javax.swing.BoxLayout(pnlInfo, javax.swing.BoxLayout.PAGE_AXIS));

        lblStatus.setFont(new java.awt.Font("微软雅黑", 0, 10)); // NOI18N
        lblStatus.setText("ARTIST");
        pnlInfo.add(lblStatus);

        lblTitle.setFont(new java.awt.Font("微软雅黑", 1, 14));
        lblTitle.setText("TITLE!!!");
        pnlInfo.add(lblTitle);

        lblExtraInfo.setFont(new java.awt.Font("微软雅黑", 0, 10));
        lblExtraInfo.setForeground(new java.awt.Color(128, 128, 128));
        lblExtraInfo.setText("jLabel1");
        pnlInfo.add(lblExtraInfo);

        pnlControls.add(pnlInfo);

        pnlPlayControls.setLayout(new javax.swing.BoxLayout(pnlPlayControls, javax.swing.BoxLayout.LINE_AXIS));
        pnlPlayControls.add(filler1);

        btnPreviousRewind.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/control-stop-180.png"))); // NOI18N
        btnPreviousRewind.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnPreviousRewindActionPerformed(evt);
            }
        });
        pnlPlayControls.add(btnPreviousRewind);

        btnPlayPause.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/controls-play.png"))); // NOI18N
        btnPlayPause.setToolTipText("Play");
        btnPlayPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnPlayPauseActionPerformed(evt);
            }
        });
        pnlPlayControls.add(btnPlayPause);

        btnNext.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/control-stop.png"))); // NOI18N
        btnNext.setToolTipText("Next song");
        btnNext.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnNextActionPerformed(evt);
            }
        });
        pnlPlayControls.add(btnNext);
        pnlPlayControls.add(filler4);

        pnlControls.add(pnlPlayControls);

        pnlMiscControls.setLayout(new javax.swing.BoxLayout(pnlMiscControls, javax.swing.BoxLayout.LINE_AXIS));
        pnlMiscControls.add(filler2);

        btnLoopMode.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/arrow-turn.png"))); // NOI18N
        btnLoopMode.setToolTipText("Play once");
        btnLoopMode.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnLoopModeActionPerformed(evt);
            }
        });
        pnlMiscControls.add(btnLoopMode);

        btnOrder.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/arrow-skip.png"))); // NOI18N
        btnOrder.setToolTipText("Straight playthrough");
        btnOrder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOrderActionPerformed(evt);
            }
        });
        pnlMiscControls.add(btnOrder);

        btnVolume.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/speaker-volume.png"))); // NOI18N
        btnVolume.setToolTipText("Volume");
        btnVolume.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnVolumeActionPerformed(evt);
            }
        });
        pnlMiscControls.add(btnVolume);

        sldVolume.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        sldVolume.setPreferredSize(new java.awt.Dimension(50, 23));
        sldVolume.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                sldVolumeStateChanged(evt);
            }
        });
        pnlMiscControls.add(sldVolume);

        pnlControls.add(pnlMiscControls);

        pnlMain.add(pnlControls);

        getContentPane().add(pnlMain, java.awt.BorderLayout.PAGE_START);

        pnlBottom.setDividerLocation(325);
        pnlBottom.setResizeWeight(0.5);

        pnlLyricsScroll.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        pnlLyricsScroll.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        pnlLyricsScroll.setMinimumSize(new java.awt.Dimension(100, 6));
        pnlLyricsScroll.setPreferredSize(new java.awt.Dimension(300, 100));

        pnlLyrics.setLayout(new javax.swing.BoxLayout(pnlLyrics, javax.swing.BoxLayout.PAGE_AXIS));
        pnlLyricsScroll.setViewportView(pnlLyrics);

        pnlBottom.setLeftComponent(pnlLyricsScroll);

        pnlServer.setMinimumSize(new java.awt.Dimension(150, 100));
        pnlServer.setPreferredSize(new java.awt.Dimension(501, 200));
        pnlServer.setLayout(new java.awt.BorderLayout());

        pnlServerControls.setLayout(new javax.swing.BoxLayout(pnlServerControls, javax.swing.BoxLayout.LINE_AXIS));

        jLabel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/server-network.png"))); // NOI18N
        jLabel1.setLabelFor(txtHost);
        jLabel1.setToolTipText("Host");
        pnlServerControls.add(jLabel1);

        txtHost.setText("127.0.0.1");
        pnlServerControls.add(txtHost);
        pnlServerControls.add(filler5);

        jLabel2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/network-hub.png"))); // NOI18N
        jLabel2.setLabelFor(txtUsername);
        jLabel2.setToolTipText("Port");
        pnlServerControls.add(jLabel2);

        txtPort.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        txtPort.setText("3000");
        pnlServerControls.add(txtPort);
        pnlServerControls.add(filler8);

        jLabel3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/user-white.png"))); // NOI18N
        jLabel3.setLabelFor(txtUsername);
        jLabel3.setToolTipText("Username");
        pnlServerControls.add(jLabel3);

        txtUsername.setText("username");
        pnlServerControls.add(txtUsername);

        btnConnect.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/plug-disconnect.png"))); // NOI18N
        btnConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConnectActionPerformed(evt);
            }
        });
        pnlServerControls.add(btnConnect);

        pnlServer.add(pnlServerControls, java.awt.BorderLayout.PAGE_START);

        tblServer.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Title", "Artist", "Album"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                false, false, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        tblServer.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        pnlServerScroll.setViewportView(tblServer);

        pnlServer.add(pnlServerScroll, java.awt.BorderLayout.CENTER);

        pnlServerListControls.setLayout(new javax.swing.BoxLayout(pnlServerListControls, javax.swing.BoxLayout.PAGE_AXIS));
        pnlServerListControls.add(filler6);

        btnServerListPrevious.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/control-stop-090.png"))); // NOI18N
        btnServerListPrevious.setToolTipText("Previous page");
        btnServerListPrevious.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnServerListPreviousActionPerformed(evt);
            }
        });
        pnlServerListControls.add(btnServerListPrevious);

        btnServerListRefresh.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/arrow-circle-double-135.png"))); // NOI18N
        btnServerListRefresh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnServerListRefreshActionPerformed(evt);
            }
        });
        pnlServerListControls.add(btnServerListRefresh);

        btnServerListNext.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/control-stop-270.png"))); // NOI18N
        btnServerListNext.setToolTipText("Next page");
        btnServerListNext.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnServerListNextActionPerformed(evt);
            }
        });
        pnlServerListControls.add(btnServerListNext);
        pnlServerListControls.add(filler9);

        btnServerListDownload.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/download-cloud.png"))); // NOI18N
        btnServerListDownload.setToolTipText("Download");
        btnServerListDownload.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnServerListDownloadActionPerformed(evt);
            }
        });
        pnlServerListControls.add(btnServerListDownload);

        btnServerListPlay.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/control.png"))); // NOI18N
        btnServerListPlay.setToolTipText("Play");
        btnServerListPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnServerListPlayActionPerformed(evt);
            }
        });
        pnlServerListControls.add(btnServerListPlay);
        pnlServerListControls.add(filler7);

        pnlServer.add(pnlServerListControls, java.awt.BorderLayout.LINE_START);

        pnlServerStatus.setLayout(new javax.swing.BoxLayout(pnlServerStatus, javax.swing.BoxLayout.LINE_AXIS));

        lblServerStatus.setText("Ready");
        lblServerStatus.setPreferredSize(new java.awt.Dimension(200, 15));
        pnlServerStatus.add(lblServerStatus);
        pnlServerStatus.add(filler10);

        jLabel4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/melodia/player/gui/resources/magnifier-left.png"))); // NOI18N
        pnlServerStatus.add(jLabel4);

        txtServerListSearch.setColumns(10);
        txtServerListSearch.getDocument().addDocumentListener(new DocumentListener() {
            @Override
            public void insertUpdate(DocumentEvent de) {
                serverSearch();
            }
            @Override
            public void removeUpdate(DocumentEvent de) {
                serverSearch();
            }
            @Override
            public void changedUpdate(DocumentEvent de) {
                serverSearch();
            }
        });
        pnlServerStatus.add(txtServerListSearch);

        pnlServer.add(pnlServerStatus, java.awt.BorderLayout.PAGE_END);

        pnlBottom.setRightComponent(pnlServer);

        getContentPane().add(pnlBottom, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void btnAddSongActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddSongActionPerformed
  if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
    for (File file : fileChooser.getSelectedFiles()) {
      playlist.addSong(new Song(file));
    }
  }
}//GEN-LAST:event_btnAddSongActionPerformed

private void btnPlayPauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnPlayPauseActionPerformed
  if (player.isPlaying()) {
    player.stop();
  } else {
    // Partial fix for the variable bitrate problem.
    // TODO: Find a way to really fix it.
    player.setProgress(getSliderPercentage(sldProgress));
    player.start();
  }
}//GEN-LAST:event_btnPlayPauseActionPerformed

private void sldVolumeStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sldVolumeStateChanged
  double perc = getSliderPercentage(sldVolume);
  player.setVolume(perc);
  if (perc < 1e-5) {
    btnVolume.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/speaker-volume-control-mute.png")));
  } else if (perc < 0.33) {
    btnVolume.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/speaker-volume-none.png")));
  } else if (perc < 0.66) {
    btnVolume.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/speaker-volume-low.png")));
  } else {
    btnVolume.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/speaker-volume.png")));
  }
}//GEN-LAST:event_sldVolumeStateChanged

private void sldProgressStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sldProgressStateChanged
  if (suppressUpdate) {
    suppressUpdate = false;
    return;
  }
  if (!sldProgress.getValueIsAdjusting()) {
    player.setProgress(getSliderPercentage(sldProgress));
  }
}//GEN-LAST:event_sldProgressStateChanged

private void btnPreviousRewindActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnPreviousRewindActionPerformed
  if (player.getCurrentTime() <= 10) {
    // If the song just started, switch to the previous song.
    player.switchSong(playlist.getPreviousSong(player.getCurrentSong(),
        loopMode, randomized));
  } else {
    // Otherwise rewind to the beginning.
    player.setProgress(0);
  }
}//GEN-LAST:event_btnPreviousRewindActionPerformed

private void btnNextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnNextActionPerformed
  player.switchSong(playlist.getNextSong(player.getCurrentSong(), loopMode,
      randomized));
}//GEN-LAST:event_btnNextActionPerformed

private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
  player.dispose();
  playlist.storeConfig();
  saveGUIConfig();
  melodia.util.Config.getInstance().writeConfigFile();
}//GEN-LAST:event_formWindowClosing

private void btnSavePlaylistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSavePlaylistActionPerformed
  playlist.savePlaylist();
}//GEN-LAST:event_btnSavePlaylistActionPerformed

  private void btnLoopModeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnLoopModeActionPerformed
    switch (loopMode) {
      case 0:       // Non-looping => Playlist loop
        btnLoopMode.setIcon(new ImageIcon(getClass().getResource(
            "/melodia/player/gui/resources/arrow-repeat.png")));
        btnLoopMode.setToolTipText("Loop playlist");
        loopMode = 1;
        break;
      case 1:       // Playlist loop => Single song loop
        btnLoopMode.setIcon(new ImageIcon(getClass().getResource(
            "/melodia/player/gui/resources/arrow-repeat-once.png")));
        btnLoopMode.setToolTipText("Loop single song");
        loopMode = 2;
        break;
      case 2:       // Single song loop => Non-looping
        btnLoopMode.setIcon(new ImageIcon(getClass().getResource(
            "/melodia/player/gui/resources/arrow-turn.png")));
        btnLoopMode.setToolTipText("Play once");
        loopMode = 0;
    }
  }//GEN-LAST:event_btnLoopModeActionPerformed

  private void btnOrderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOrderActionPerformed
    randomized = !randomized;
    btnOrder.setIcon(new ImageIcon(getClass().getResource("/melodia/player/gui"
        + "/resources/arrow-" + (randomized ? "switch" : "skip") + ".png")));
    btnOrder.setToolTipText(randomized ? "Shuffle playlist" : "Straight playthrough");
  }//GEN-LAST:event_btnOrderActionPerformed

  private void tlgPlaylistItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_tlgPlaylistItemStateChanged
    if (evt.getStateChange() == ItemEvent.SELECTED) {
      pnlTop.setVisible(true);
    } else {
      pnlTop.setVisible(false);
    }
    pack();
  }//GEN-LAST:event_tlgPlaylistItemStateChanged

  private void btnClearPlaylistActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnClearPlaylistActionPerformed
    playlist.clearPlaylist();
  }//GEN-LAST:event_btnClearPlaylistActionPerformed

  private void btnMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMenuActionPerformed
    popupMenu.show(btnMenu, 0, btnMenu.getHeight());
  }//GEN-LAST:event_btnMenuActionPerformed

  private void btnCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCloseActionPerformed
    dispose();
  }//GEN-LAST:event_btnCloseActionPerformed

  private void btnMinimizeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMinimizeActionPerformed
    setState(ICONIFIED);
  }//GEN-LAST:event_btnMinimizeActionPerformed

  private void tlgLyricsItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_tlgLyricsItemStateChanged
    if (evt.getStateChange() == ItemEvent.SELECTED) {
      pnlLyricsScroll.setVisible(true);
      pnlBottom.resetToPreferredSizes();
      pnlBottom.setVisible(true);
    } else {
      pnlLyricsScroll.setVisible(false);
      if (!pnlServer.isVisible()) pnlBottom.setVisible(false);
    }
    pack();
  }//GEN-LAST:event_tlgLyricsItemStateChanged

  private void tlgServerItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_tlgServerItemStateChanged
    if (evt.getStateChange() == ItemEvent.SELECTED) {
      pnlServer.setVisible(true);
      pnlBottom.resetToPreferredSizes();
      pnlBottom.setVisible(true);
    } else {
      pnlServer.setVisible(false);
      if (!pnlLyricsScroll.isVisible()) pnlBottom.setVisible(false);
    }
    pack();
  }//GEN-LAST:event_tlgServerItemStateChanged

  private void formWindowClosed(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosed
    formWindowClosing(evt);
    System.exit(0);
  }//GEN-LAST:event_formWindowClosed

  private void menuAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuAboutActionPerformed
    JOptionPane.showMessageDialog(this, "<html><p><b>Melodia Player</b></p><p>by Wyverald</p>"
        + "<p>Fugue icons by Yusuke Kamiyamane</p><p><a href=\""
        + "http://p.yusukekamiyamane.com/\">http://p.yusukekamiyamane.com/</a></p></html>",
        "About Melodia", JOptionPane.INFORMATION_MESSAGE);
  }//GEN-LAST:event_menuAboutActionPerformed

  private void btnVolumeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnVolumeActionPerformed
    if (getSliderPercentage(sldVolume) < 1e-5) {
      setSliderPercentage(sldVolume, 0.3);
    } else {
      setSliderPercentage(sldVolume, 0);
    }
  }//GEN-LAST:event_btnVolumeActionPerformed

  private void btnConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConnectActionPerformed
    if (connected) {
      setConnected(false);
    } else {
      try {
        serverLoading("Connecting to server...");
        RequestHandler request = new RequestHandler(this, tblServer.getModel(),
            new ListRequest(), txtHost.getText(), Integer.parseInt(txtPort.getText()));
        request.execute();
        setConnected(true);
      } catch (UnknownHostException ex) {
        serverError("Cannot find host.");
      } catch (IOException ex) {
        serverError("Cannot connect to server.");
      }
    }
  }//GEN-LAST:event_btnConnectActionPerformed

  private void btnServerListRefreshActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnServerListRefreshActionPerformed
    try {
      serverLoading("Refreshing list...");
      new RequestHandler(this, tblServer.getModel(), new ListRequest(
          txtServerListSearch.getText(), serverListOffset, ListRequest.DEFAULT_PAGE_SIZE),
          txtHost.getText(), Integer.parseInt(txtPort.getText())).execute();
    } catch (UnknownHostException ex) {
      serverError("Cannot find host.");
    } catch (IOException ex) {
      serverError("Cannot connect to server.");
    }
  }//GEN-LAST:event_btnServerListRefreshActionPerformed

  private void btnServerListPreviousActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnServerListPreviousActionPerformed
    try {
      serverLoading("Refreshing list...");
      new RequestHandler(this, tblServer.getModel(), new ListRequest(
          txtServerListSearch.getText(), Math.max(serverListOffset - 
          ListRequest.DEFAULT_PAGE_SIZE, 1), ListRequest.DEFAULT_PAGE_SIZE),
          txtHost.getText(), Integer.parseInt(txtPort.getText())).execute();
    } catch (UnknownHostException ex) {
      serverError("Cannot find host.");
    } catch (IOException ex) {
      serverError("Cannot connect to server.");
    }
  }//GEN-LAST:event_btnServerListPreviousActionPerformed

  private void btnServerListNextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnServerListNextActionPerformed
    try {
      serverLoading("Refreshing list...");
      new RequestHandler(this, tblServer.getModel(), new ListRequest(
          txtServerListSearch.getText(), serverListOffset + 
          ListRequest.DEFAULT_PAGE_SIZE, ListRequest.DEFAULT_PAGE_SIZE),
          txtHost.getText(), Integer.parseInt(txtPort.getText())).execute();
    } catch (UnknownHostException ex) {
      serverError("Cannot find host.");
    } catch (IOException ex) {
      serverError("Cannot connect to server.");
    }
  }//GEN-LAST:event_btnServerListNextActionPerformed

  private void btnServerListDownloadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnServerListDownloadActionPerformed
    if (tblServer.getSelectedRow() < 0) {
      serverError("No song selected.");
    } else {
      // Select destination file.
      if (fileChooser.showSaveDialog(this) != JFileChooser.APPROVE_OPTION) return;
      int row = tblServer.getSelectedRow();
      javax.swing.table.TableModel tm = tblServer.getModel();
      try {
        serverLoading("Downloading file...");
        new RequestHandler(this, tblServer.getModel(), new DownloadRequest(
            fileChooser.getSelectedFile(), tm.getValueAt(row, 0).toString(),
            tm.getValueAt(row, 1).toString(), tm.getValueAt(row, 2).toString()),
            txtHost.getText(), Integer.parseInt(txtPort.getText())).execute();
      } catch (UnknownHostException ex) {
        serverError("Cannot find host.");
      } catch (IOException ex) {
        serverError("Cannot connect to server.");
      }
      
    }
  }//GEN-LAST:event_btnServerListDownloadActionPerformed

  private void btnServerListPlayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnServerListPlayActionPerformed
    if (tblServer.getSelectedRow() < 0) {
      serverError("No song selected.");
    } else {
      int row = tblServer.getSelectedRow();
      javax.swing.table.TableModel tm = tblServer.getModel();
      try {
        serverLoading("Downloading file...");
        new RequestHandler(this, tblServer.getModel(), new DownloadAndPlayRequest(
            File.createTempFile("melodia", ".mp3"), tm.getValueAt(row, 0).toString(),
            tm.getValueAt(row, 1).toString(), tm.getValueAt(row, 2).toString()),
            txtHost.getText(), Integer.parseInt(txtPort.getText())).execute();
      } catch (UnknownHostException ex) {
        serverError("Cannot find host.");
      } catch (IOException ex) {
        serverError("Cannot connect to server.");
      }
      
    }
  }//GEN-LAST:event_btnServerListPlayActionPerformed

  private void menuMiniModeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuMiniModeActionPerformed
    tlgServer.setSelected(false);
    tlgPlaylist.setSelected(false);
    tlgLyrics.setSelected(false);
  }//GEN-LAST:event_menuMiniModeActionPerformed

  private void menuExpandedActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuExpandedActionPerformed
    tlgServer.setSelected(true);
    tlgPlaylist.setSelected(true);
    tlgLyrics.setSelected(true);
  }//GEN-LAST:event_menuExpandedActionPerformed

  private void menuAlwaysOnTopItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_menuAlwaysOnTopItemStateChanged
    setAlwaysOnTop(evt.getStateChange() == ItemEvent.SELECTED);
  }//GEN-LAST:event_menuAlwaysOnTopItemStateChanged

  /* UI modifications */
  private void replaceDivider() {
    Container divider = null;
    for (int i = 0; i < pnlTop.getComponentCount(); i++) {
      if (!(pnlTop.getComponent(i) instanceof JScrollPane)) {
        divider = (Container) pnlTop.getComponent(i);
        break;
      }
    }
    divider.setLayout(new BoxLayout(divider, BoxLayout.PAGE_AXIS));
    divider.add(btnAddSong);
    divider.add(btnClearPlaylist);
    divider.add(Box.createVerticalGlue());
    divider.add(btnSavePlaylist);
  }
  
  private void makeHoverButtons() {
    JButton[] buttons = new JButton[] {
      btnPlayPause, btnPreviousRewind, btnNext, btnLoopMode, btnOrder,
      btnAddSong, btnClearPlaylist, btnSavePlaylist, btnVolume
    };
    for (JButton btn : buttons) {
      btn.setContentAreaFilled(false);
      btn.addMouseListener(HoverButtonMouseListener.getInstance());
    }
  }
  
  private void decorateWindow() {
    // Add a border around the root pane.
    getRootPane().setBorder(BorderFactory.createRaisedBevelBorder());
    
    // Allow the user to drag the window around.
    MouseInputAdapter mia1 = new MouseInputAdapter() {
      int diffX, diffY;

      @Override
      public void mouseDragged(MouseEvent me) {
        Point loc = me.getLocationOnScreen();
        loc.translate(diffX, diffY);
        MainFrame.this.setLocation(loc);
      }

      @Override
      public void mousePressed(MouseEvent me) {
        Point mouseLocation = me.getLocationOnScreen();
        Point windowLocation = MainFrame.this.getLocation();
        diffX = windowLocation.x - mouseLocation.x;
        diffY = windowLocation.y - mouseLocation.y;
      }
    };
    pnlTitleBar.addMouseMotionListener(mia1);
    pnlTitleBar.addMouseListener(mia1);
    
    // Allow the user to resize the window.
    MouseInputAdapter mia2 = new MouseInputAdapter() {
      int diffX, diffY, seX, seY;
      
      @Override
      public void mouseDragged(MouseEvent me) {
        Point loc = me.getLocationOnScreen();
        MainFrame.this.setBounds(loc.x + diffX, loc.y + diffY, seX - loc.x - diffX, 
            pnlBottom.isVisible() ? seY - loc.y - diffY : MainFrame.this.getHeight());
      }
      
      @Override
      public void mousePressed(MouseEvent me) {
        Point mouseLocation = me.getLocationOnScreen();
        Point windowLocation = MainFrame.this.getLocationOnScreen();
        Dimension windowSize = MainFrame.this.getSize();
        diffX = windowLocation.x - mouseLocation.x;
        diffY = windowLocation.y - mouseLocation.y;
        seX = windowLocation.x + windowSize.width;
        seY = windowLocation.y + windowSize.height;
      }
    };
    lblResize.addMouseMotionListener(mia2);
    lblResize.addMouseListener(mia2);
  }
  
  private void createFileChooser() {
    fileChooser = new JFileChooser();
    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fileChooser.setMultiSelectionEnabled(true);
    javax.swing.filechooser.FileFilter ff = new javax.swing.filechooser.FileFilter() {

      @Override
      public boolean accept(File file) {
        return file.getName().endsWith(".mp3") || file.isDirectory();
      }

      @Override
      public String getDescription() {
        return "Music files";
      }
    };
    fileChooser.setFileFilter(ff);
  }
  
  private void setConnected(boolean con) {
    connected = con;
    for (Component comp : pnlPlaylist.getComponents()) {
      if (comp instanceof SongButton) {
        SongButton songButton = (SongButton) comp;
        songButton.getUploadButton().setVisible(con);
      }
    }
    btnConnect.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/plug-disconnect" + 
        (con ? "-slash.png" : ".png"))));
    
    txtHost.setEnabled(!con);
    txtPort.setEnabled(!con);
    txtUsername.setEnabled(!con);
    btnServerListRefresh.setEnabled(con);
    btnServerListPrevious.setEnabled(con);
    btnServerListNext.setEnabled(con);
    btnServerListDownload.setEnabled(con);
    btnServerListPlay.setEnabled(con);
    txtServerListSearch.setEnabled(con);
    updateListPageButtons();
  }
  
  /* Configuration loading and saving */
  private void loadGUIConfig() {
    JSONObject jo = null;
    try {
      jo = Config.getInstance().getConfigObject().getJSONObject("gui");
    } catch (JSONException ex) {
      return;
    }
    
    try {
      tlgLyrics.setSelected(jo.getBoolean("lyricsPane"));
      tlgServer.setSelected(jo.getBoolean("serverPane"));
      tlgPlaylist.setSelected(jo.getBoolean("playlistPane"));
      
      setSliderPercentage(sldVolume, jo.getDouble("volume"));
      int loop = jo.getInt("loopMode");
      for (int i = 0; i < loop && i < 2; i++) btnLoopMode.doClick();
      if (jo.getBoolean("random")) btnOrder.doClick();
    } catch (JSONException ex) {
      System.err.println("Error reading GUI config");
    }
  }
  
  private void saveGUIConfig() {
    JSONObject jo = new JSONObject();
    try {
      jo.put("lyricsPane", tlgLyrics.isSelected());
      jo.put("serverPane", tlgServer.isSelected());
      jo.put("playlistPane", tlgPlaylist.isSelected());
      
      jo.put("volume", getSliderPercentage(sldVolume));
      jo.put("loopMode", loopMode);
      jo.put("random", randomized);
      
      Config.getInstance().getConfigObject().put("gui", jo);
    } catch (JSONException ex) {
      System.err.println("Error writing GUI config");
    }
  }
  
  /* Timer events */
  private void progressTimerTick(ActionEvent ae) {
    // Update the progress bar and time indicator.
    double curTime = player.getCurrentTime();
    if (!sldProgress.getValueIsAdjusting()) {
      suppressUpdate = true;
      setSliderPercentage(sldProgress, player.getProgress());
      lblTime.setText(PlayerManager.formatTime(curTime));
    } else {
      lblTime.setText(PlayerManager.formatTime(getSliderPercentage(sldProgress)
          * player.getDuration()));
    }

    // Update lyrics.
    lyricsScroller.update(player.getCurrentTime());

    // Update "previous" button label.
    if (curTime > 10) {
      btnPreviousRewind.setIcon(new ImageIcon(getClass().getResource(
          "/melodia/player/gui/resources/control-double-180.png")));
      btnPreviousRewind.setToolTipText("Rewind");
    } else {
      btnPreviousRewind.setIcon(new ImageIcon(getClass().getResource(
          "/melodia/player/gui/resources/control-stop-180.png")));
      btnPreviousRewind.setToolTipText("Previous song");
    }
  }

  private void statusTimerTick() {
    if (player.getCurrentSong() == null) {
      lblStatus.setText("");
      lblTitle.setText("");
      lblExtraInfo.setText("");
      return;
    }

    SongMetadata metadata = player.getCurrentSong().getMetadata();
    lblStatus.setText(player.isPlaying() ? "Now playing" : "Current song");
    lblTitle.setText(metadata.getTitle());

    switch (statusType) {
      case 0:
        lblExtraInfo.setText("From: " + metadata.getAlbum());
        break;
      case 1:
        lblExtraInfo.setText("By: " + metadata.getArtist());
        break;
      case 2:
        lblExtraInfo.setText("Year: " + metadata.getYear());
        break;
      case 3:
        lblExtraInfo.setText("Track: " + metadata.getTrackNumber());
        break;
      case 4:
        lblExtraInfo.setText("Genre: " + metadata.getGenre());
        break;
    }
  }

  /* These are static methods that should be refactored into member methods
   * once we're done with the layout.
   */
  public static double getSliderPercentage(JSlider sld) {
    return (double) sld.getValue() / (sld.getMaximum() - sld.getMinimum());
  }

  public static void setSliderPercentage(JSlider sld, double perc) {
    sld.setValue(sld.getMinimum() + (int) (perc * (sld.getMaximum()
        - sld.getMinimum())));
  }
  
  /* Convenience methods to update the server status. */
  public void serverInfo(String message) {
    lblServerStatus.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/information.png")));
    lblServerStatus.setText(message);
  }
  
  public void serverLoading(String message) {
    lblServerStatus.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/hourglass.png")));
    lblServerStatus.setText(message);
  }
  
  public void serverError(String message) {
    lblServerStatus.setIcon(new ImageIcon(getClass().getResource(
        "/melodia/player/gui/resources/cross-circle-small.png")));
    lblServerStatus.setText(message);
  }
  
  public void setListPages(int offset, int total) {
    this.serverListOffset = offset;
    this.serverListTotal = total;
    updateListPageButtons();
  }
  
  public void updateListPageButtons() {
    btnServerListPrevious.setEnabled(serverListOffset > 1);
    btnServerListNext.setEnabled(serverListOffset + ListRequest.DEFAULT_PAGE_SIZE <= serverListTotal);
  }
  
  public void serverSearch() {
    try {
      serverLoading("Searching...");
      new RequestHandler(this, tblServer.getModel(), new ListRequest(
          txtServerListSearch.getText()),
          txtHost.getText(), Integer.parseInt(txtPort.getText())).execute();
    } catch (UnknownHostException ex) {
      serverError("Cannot find host.");
    } catch (IOException ex) {
      serverError("Cannot connect to server.");
    }
  }
  
  public void serverRefresh() {
    btnServerListRefresh.doClick();
  }
  
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAddSong;
    private javax.swing.JButton btnClearPlaylist;
    private javax.swing.JButton btnClose;
    private javax.swing.JButton btnConnect;
    private javax.swing.JButton btnLoopMode;
    private javax.swing.JButton btnMenu;
    private javax.swing.JButton btnMinimize;
    private javax.swing.JButton btnNext;
    private javax.swing.JButton btnOrder;
    private javax.swing.JButton btnPlayPause;
    private javax.swing.JButton btnPreviousRewind;
    private javax.swing.JButton btnSavePlaylist;
    private javax.swing.JButton btnServerListDownload;
    private javax.swing.JButton btnServerListNext;
    private javax.swing.JButton btnServerListPlay;
    private javax.swing.JButton btnServerListPrevious;
    private javax.swing.JButton btnServerListRefresh;
    private javax.swing.JButton btnVolume;
    private javax.swing.Box.Filler filler1;
    private javax.swing.Box.Filler filler10;
    private javax.swing.Box.Filler filler11;
    private javax.swing.Box.Filler filler2;
    private javax.swing.Box.Filler filler3;
    private javax.swing.Box.Filler filler4;
    private javax.swing.Box.Filler filler5;
    private javax.swing.Box.Filler filler6;
    private javax.swing.Box.Filler filler7;
    private javax.swing.Box.Filler filler8;
    private javax.swing.Box.Filler filler9;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel lblAppTitle;
    private javax.swing.JLabel lblDuration;
    private javax.swing.JLabel lblExtraInfo;
    private javax.swing.JLabel lblResize;
    private javax.swing.JLabel lblServerStatus;
    private javax.swing.JLabel lblStatus;
    private javax.swing.JLabel lblTime;
    private javax.swing.JLabel lblTitle;
    private javax.swing.JLabel lblTitleBackground;
    private javax.swing.JMenuItem menuAbout;
    private javax.swing.JCheckBoxMenuItem menuAlwaysOnTop;
    private javax.swing.JMenuItem menuExpanded;
    private javax.swing.JMenuItem menuMiniMode;
    private javax.swing.JSplitPane pnlBottom;
    private javax.swing.JPanel pnlControls;
    private javax.swing.JPanel pnlInfo;
    private javax.swing.JPanel pnlLyrics;
    private javax.swing.JScrollPane pnlLyricsScroll;
    private javax.swing.JPanel pnlMain;
    private javax.swing.JPanel pnlMiscControls;
    private javax.swing.JPanel pnlPlayControls;
    private javax.swing.JPanel pnlPlaylist;
    private javax.swing.JScrollPane pnlPlaylistScroll;
    private javax.swing.JPanel pnlPlaylists;
    private javax.swing.JScrollPane pnlPlaylistsScroll;
    private javax.swing.JPanel pnlProgress;
    private javax.swing.JPanel pnlServer;
    private javax.swing.JPanel pnlServerControls;
    private javax.swing.JPanel pnlServerListControls;
    private javax.swing.JScrollPane pnlServerScroll;
    private javax.swing.JPanel pnlServerStatus;
    private javax.swing.JPanel pnlTitleBar;
    private javax.swing.JPanel pnlTitleBarContainer;
    private javax.swing.JSplitPane pnlTop;
    private javax.swing.JPanel pnlView;
    private javax.swing.JPanel pnlWindowControls;
    private javax.swing.JPopupMenu popupMenu;
    private javax.swing.JSlider sldProgress;
    private javax.swing.JSlider sldVolume;
    private javax.swing.JTable tblServer;
    private javax.swing.JToggleButton tlgLyrics;
    private javax.swing.JToggleButton tlgPlaylist;
    private javax.swing.JToggleButton tlgServer;
    private javax.swing.JTextField txtHost;
    private javax.swing.JFormattedTextField txtPort;
    private javax.swing.JTextField txtServerListSearch;
    private javax.swing.JTextField txtUsername;
    // End of variables declaration//GEN-END:variables

  /* Player event listeners */
  @Override
  public void beforeNewPlayerCreated(Song song) {
    //btnPlayPause.setText("Loading...");
    setSliderPercentage(sldProgress, 0);
    if (song == null) return;

    pnlLyrics.removeAll();
    lyricsScroller = new LyricsScroller(pnlLyricsScroll, pnlLyrics,
        song.getLyrics());

    song.getButton().setHighlight(true);
    pnlPlaylist.validate();
    pnlPlaylist.scrollRectToVisible(song.getButton().getBounds());
  }

  @Override
  public void playerStopped() {
    progressTimer.stop();
    btnPlayPause.setIcon(new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/controls-play.png")));
    btnPlayPause.setToolTipText("Play");
    statusTimerTick();
  }

  @Override
  public void playerStarted() {
    progressTimer.start();
    btnPlayPause.setIcon(new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/controls-pause.png")));
    btnPlayPause.setToolTipText("Pause");
    player.setVolume(getSliderPercentage(sldVolume));
    statusTimerTick();
  }

  @Override
  public void durationUpdated() {
    lblDuration.setText(PlayerManager.formatTime(player.getDuration()));
  }

  @Override
  public void endOfMediaReached(Song song) {
    btnNextActionPerformed(null);
  }

  @Override
  public void oldPlayerDisposed(Song song) {
    song.getButton().setHighlight(false);
    lblDuration.setText(PlayerManager.formatTime(0));
    lblTime.setText(PlayerManager.formatTime(0));
    setSliderPercentage(sldProgress, 0);
    pnlLyrics.removeAll();
    statusTimerTick();
  }

  /* Playlist event listeners */
  @Override
  public void songAdded(Song song) {
    if (pnlPlaylist.getComponentCount() == 0) {
      player.switchSong(song);
    }
    pnlPlaylist.add(song.getButton());
    if (!connected) song.getButton().getUploadButton().setVisible(false);
    song.getButton().addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent ae) {
        player.switchSong(((SongButton) ae.getSource()).getSong());
      }
    });
    song.getButton().getRemoveButton().addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent ae) {
        SubButton sub = (SubButton) ae.getSource();
        SongButton sb = (SongButton) sub.getParentButton();
        if (player.getCurrentSong() == sb.getSong()) {
          if (playlist.getNumberOfSongs() == 1) {
            player.dispose();
          } else {
            btnNextActionPerformed(null);
          }
        }
        playlist.removeSong(sb.getSong());
      }
    });
    song.getButton().getUploadButton().addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent ae) {
        if (!connected) return;
        tlgServer.setSelected(true);
        
        SubButton sub = (SubButton) ae.getSource();
        SongButton sb = (SongButton) sub.getParentButton();
        Song song = sb.getSong();
        SongMetadata data = song.getMetadata();
        
        try {
          serverLoading("Uploading...");
          new RequestHandler(MainFrame.this, tblServer.getModel(), new UploadRequest(
              song.getFile(), data.getTitle(), data.getArtist(), data.getAlbum()),
              txtHost.getText(), Integer.parseInt(txtPort.getText())).execute();
        } catch (UnknownHostException ex) {
          serverError("Cannot find host.");
        } catch (IOException ex) {
          serverError("Cannot connect to server.");
        }
      }
    });
  }

  @Override
  public void songRemoved(Song song) {
    pnlPlaylist.remove(song.getButton());
    pnlPlaylistScroll.validate();
  }

  @Override
  public void playlistCleared() {
    player.dispose();
    pnlPlaylist.removeAll();
    pnlPlaylistScroll.validate();
  }

  @Override
  public void playlistAdded(Playlist playlist) {
    pnlPlaylists.add(playlist.getButton());
    playlist.getButton().getRemoveButton().addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent ae) {
        SubButton sb = (SubButton) ae.getSource();
        PlaylistButton pb = (PlaylistButton) sb.getParentButton();
        MainFrame.this.playlist.removeSavedPlaylist(pb.getPlaylist());
      }
    });
    playlist.getButton().getAddButton().addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent ae) {
        SubButton sb = (SubButton) ae.getSource();
        PlaylistButton pb = (PlaylistButton) sb.getParentButton();
        MainFrame.this.playlist.appendSavedPlaylist(pb.getPlaylist());
      }
    });
    playlist.getButton().getPlayButton().addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent ae) {
        SubButton sb = (SubButton) ae.getSource();
        PlaylistButton pb = (PlaylistButton) sb.getParentButton();
        MainFrame.this.playlist.playSavedPlaylist(pb.getPlaylist());
      }
    });
    playlist.getButton().getRenameButton().addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent ae) {
        SubButton sb = (SubButton) ae.getSource();
        PlaylistButton pb = (PlaylistButton) sb.getParentButton();
        MainFrame.this.playlist.renameSavedPlaylist(pb.getPlaylist());
      }
    });
  }

  @Override
  public void playlistRemoved(Playlist playlist) {
    pnlPlaylists.remove(playlist.getButton());
    pnlPlaylistsScroll.validate();
  }
}