/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player.gui;

import java.awt.event.MouseEvent;
import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseListener;
import melodia.player.Song;
import melodia.player.SongMetadata;

/**
 *
 * @author Wyverald
 */
public class SongButton extends ListButton implements MouseListener {
  private Song song;
  private JLabel lblTitle, lblArtist, lblCoverArt;
  private ImageIcon biggerIcon, smallerIcon;
  private JPanel pnlInfo, pnlSubInfo, pnlButtons;
  private SubButton btnUpload, btnRemove;
  
  private final static Dimension lowlightDimension = new Dimension(95, 135);
  private final static Dimension highlightDimension = new Dimension(270, 135);

  @SuppressWarnings("OverridableMethodCallInConstructor")
  public SongButton(Song song) {
    this.song = song;
    setAlignmentY(0.5f);
    
    // Construct the button components with metadata.
    SongMetadata data = song.getMetadata();
    java.awt.Image coverArt = data.getCoverArt();
    biggerIcon = new ImageIcon(coverArt.getScaledInstance(120, 120,
        Image.SCALE_AREA_AVERAGING));
    smallerIcon = new ImageIcon(coverArt.getScaledInstance(80, 80,
        Image.SCALE_AREA_AVERAGING));
    lblCoverArt = new JLabel(smallerIcon);
    lblCoverArt.setAlignmentX(0);
    lblCoverArt.setAlignmentY(0);
    lblTitle = new JLabel(data.getTitle());
    lblTitle.setAlignmentX(0);
    lblTitle.setFont(new Font("Microsoft YaHei", Font.PLAIN, 15));
    if (song.isTemporary()) {
      lblTitle.setForeground(Color.BLUE);
      lblTitle.setToolTipText("This song is played directly from the server.");
    }
    lblArtist = new JLabel(data.getArtist());
    lblArtist.setAlignmentX(0);
    lblArtist.setFont(new Font("Microsoft YaHei", Font.PLAIN, 12));
    lblArtist.setForeground(Color.GRAY);
    
    // When song button is hovered on, this panel will be reused.
    pnlButtons = new JPanel();
    pnlButtons.setLayout(new BoxLayout(pnlButtons, BoxLayout.LINE_AXIS));
    pnlButtons.setOpaque(false);
    pnlButtons.setAlignmentX(0);
    pnlButtons.setAlignmentY(1);
    pnlButtons.add(Box.createHorizontalGlue());
    btnUpload = new SubButton(this, "Upload this song", new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/upload-cloud.png")));
    btnRemove = new SubButton(this, "Remove this song from the playlist", new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/cross-script.png")));
    if (!song.isTemporary()) pnlButtons.add(btnUpload);
    pnlButtons.add(btnRemove);
    
    // The info panel is only displayed when the button is on highlight.
    pnlInfo = new JPanel();
    pnlInfo.setLayout(new BoxLayout(pnlInfo, BoxLayout.PAGE_AXIS));
    pnlInfo.setOpaque(false);
    pnlInfo.setMaximumSize(new Dimension(120, 135));
    pnlInfo.setPreferredSize(new Dimension(120, 135));
    pnlInfo.setAlignmentY(0);
    pnlSubInfo = new JPanel();
    pnlSubInfo.setLayout(new BoxLayout(pnlSubInfo, BoxLayout.LINE_AXIS));
    pnlSubInfo.setOpaque(false);
    pnlSubInfo.setAlignmentX(0);
    JPanel pnlSubInfoLeft = new JPanel();
    pnlSubInfoLeft.setLayout(new BoxLayout(pnlSubInfoLeft, BoxLayout.PAGE_AXIS));
    pnlSubInfoLeft.setOpaque(false);
    pnlSubInfoLeft.setAlignmentY(1);
    JLabel lblInfoTitle = new JLabel(data.getTitle());
    lblInfoTitle.setAlignmentX(0);
    lblInfoTitle.setFont(new Font("Microsoft YaHei", Font.BOLD, 15));
    if (song.isTemporary()) {
      lblInfoTitle.setForeground(Color.BLUE);
      lblInfoTitle.setToolTipText("This song is played directly from the server.");
    }
    JLabel lblInfoArtist = new JLabel(data.getArtist());
    lblInfoArtist.setAlignmentX(0);
    lblInfoArtist.setFont(new Font("Microsoft YaHei", Font.PLAIN, 12));
    JLabel lblInfoAlbum = new JLabel(data.getAlbum());
    lblInfoAlbum.setFont(new Font("Microsoft YaHei", Font.PLAIN, 12));
    lblInfoAlbum.setAlignmentX(0);
    Font smallFont = new Font("Microsoft YaHei", Font.PLAIN, 10);
    JLabel lblYear = new JLabel("Year: " + data.getYear());
    lblYear.setAlignmentX(0);
    lblYear.setFont(smallFont);
    lblYear.setForeground(Color.GRAY);
    JLabel lblTrack = new JLabel("Track no.: " + data.getTrackNumber());
    lblTrack.setAlignmentX(0);
    lblTrack.setFont(smallFont);
    lblTrack.setForeground(Color.GRAY);
    JLabel lblGenre = new JLabel("Genre: " + data.getGenre());
    lblGenre.setAlignmentX(0);
    lblGenre.setFont(smallFont);
    lblGenre.setForeground(Color.GRAY);
    pnlInfo.add(lblInfoTitle);
    pnlInfo.add(Box.createRigidArea(new Dimension(0, 5)));
    pnlInfo.add(lblInfoArtist);
    pnlInfo.add(lblInfoAlbum);
    pnlInfo.add(Box.createVerticalGlue());
    pnlInfo.add(lblGenre);
    pnlSubInfoLeft.add(lblTrack);
    pnlSubInfoLeft.add(lblYear);
    pnlSubInfo.add(pnlSubInfoLeft);
    pnlInfo.add(pnlSubInfo);
    
    setHighlight(false);
    
    addMouseListener(this);
  }
  
  @Override
  public void setHighlight(boolean highlight) {
    super.setHighlight(highlight);
    if (highlight) {
      setActualSize(highlightDimension);
      
      // Reorganize layout.
      removeAll();
      setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
      lblCoverArt.setIcon(biggerIcon);
      pnlSubInfo.add(pnlButtons);
      pnlButtons.setVisible(true);
      add(lblCoverArt);
      add(Box.createRigidArea(new Dimension(10, 0)));
      add(pnlInfo);
    } else {
      setActualSize(lowlightDimension);
      
      // Reorganize layout.
      removeAll();
      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
      lblCoverArt.setIcon(smallerIcon);
      add(lblCoverArt);
      add(lblTitle);
      add(lblArtist);
      add(pnlButtons);
      pnlButtons.setVisible(false);
    }
  }
  
  public Song getSong() { return song; }
  
  public SubButton getRemoveButton() {
    return btnRemove;
  }
  
  public SubButton getUploadButton() {
    return btnUpload;
  }

  @Override
  public void mouseClicked(MouseEvent me) {
  }

  @Override
  public void mousePressed(MouseEvent me) {
  }

  @Override
  public void mouseReleased(MouseEvent me) {
  }

  @Override
  public void mouseEntered(MouseEvent me) {
    lblArtist.setVisible(getHighlight());
    pnlButtons.setVisible(true);
  }

  @Override
  public void mouseExited(MouseEvent me) {
    // Check if the pointer has really exited the button.
    // We have to do this because mousing over inner buttons also triggers
    // this event.
    Point p = me.getPoint();
    Dimension d = getSize();
    if (p.x >= d.width || p.y >= d.height || p.x < 0 || p.y < 0) {
      lblArtist.setVisible(true);
      pnlButtons.setVisible(getHighlight());
    }
  }
  
}
